@@ -810,7 +810,8 @@ class VTKMeshIO(MeshIO):
             elif (((nr == sym) or (nr == (dim * dim))) and (nc == 1)) \
                      or ((nr == dim) and (nc == dim)):
                 fd.write('\nTENSORS %s float\n' % key)
-                data = val.data.squeeze()
+                data = val.data[:, 0, ...]
+                data.shape = (data.shape[0], -1)
                 aux = _reshape_tensors(data, dim, sym, nr)
                 _write_tensors(aux)
 
@@ -1395,7 +1396,7 @@ class HDF5MeshIO(MeshIO):
             raise ValueError('pytables not imported!')
 
         step = get_default_attr(ts, 'step', 0)
-        if step == 0:
+        if (step == 0) or not op.exists(filename):
             # A new file.
             with pt.open_file(filename, mode="w",
                               title="SfePy output file") as fd:
@@ -1417,7 +1418,7 @@ class HDF5MeshIO(MeshIO):
                                 'file closing time')
 
                 fd.create_array(fd.root, 'last_step',
-                                nm.array([0], dtype=nm.int32),
+                                nm.array([step], dtype=nm.int32),
                                 'last saved step')
 
         if out is not None:
@@ -1429,7 +1430,13 @@ class HDF5MeshIO(MeshIO):
             # Existing file.
             fd = pt.open_file(filename, mode="r+")
 
-            step_group = fd.create_group('/', 'step%d' % step, 'time step data')
+            step_group_name = 'step%d' % step
+            if step_group_name in fd.root:
+                raise ValueError('step %d is already saved in "%s" file!'
+                                 ' Possible help: remove the old file or'
+                                 ' start saving from the initial time.'
+                                 % (step, filename))
+            step_group = fd.create_group('/', step_group_name, 'time step data')
 
             ts_group = fd.create_group(step_group, 'ts', 'time stepper')
             fd.create_array(ts_group, 'step', step, 'step')
@@ -1502,6 +1509,11 @@ class HDF5MeshIO(MeshIO):
 
         return out
 
+    def _get_step_group_names(self, fd):
+        return sorted([name for name in fd.root._v_groups.keys()
+                       if name.startswith('step')],
+                      key=lambda name: int(name[4:]))
+
     def read_times(self, filename=None):
         """
         Read true time step data from individual time steps.
@@ -1518,13 +1530,13 @@ class HDF5MeshIO(MeshIO):
         filename = get_default(filename, self.filename)
         fd = pt.open_file(filename, mode='r')
 
-        steps = sorted(int(name[4:]) for name in fd.root._v_groups.keys()
-                       if name.startswith('step'))
+        steps = []
         times = []
         nts = []
-        for step in steps:
-            ts_group = fd.get_node(fd.root, 'step%d/ts' % step)
+        for gr_name in self._get_step_group_names(fd):
+            ts_group = fd.get_node(fd.root, gr_name + '/ts')
 
+            steps.append(ts_group.step.read())
             times.append(ts_group.t.read())
             nts.append(ts_group.nt.read())
         fd.close()
@@ -1539,6 +1551,9 @@ class HDF5MeshIO(MeshIO):
         filename = get_default(filename, self.filename)
         fd = pt.open_file(filename, mode="r")
 
+        if step is None:
+            step = int(self._get_step_group_names(fd)[0][4:])
+
         gr_name = 'step%d' % step
         try:
             step_group = fd.get_node(fd.root, gr_name)
@@ -1591,7 +1606,7 @@ class HDF5MeshIO(MeshIO):
 
         return out
 
-    def read_data_header(self, dname, step=0, filename=None):
+    def read_data_header(self, dname, step=None, filename=None):
         fd, step_group = self._get_step_group(step, filename=filename)
         if fd is None: return None
 
@@ -1616,9 +1631,7 @@ class HDF5MeshIO(MeshIO):
         fd = pt.open_file(filename, mode="r")
 
         th = dict_from_keys_init(indx, list)
-        for step in range(fd.root.last_step[0] + 1):
-            gr_name = 'step%d' % step
-
+        for gr_name in self._get_step_group_names(fd):
             step_group = fd.get_node(fd.root, gr_name)
             data = step_group._f_get_child(node_name).data
 
